/******************************************************************************

  Copyright(c) 2003 - 2007 Intel Corporation. All rights reserved.

  This program is free software; you can redistribute it and/or modify it
  under the terms of version 2 of the GNU General Public License as
  published by the Free Software Foundation.

  This program is distributed in the hope that it will be useful, but WITHOUT
  ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
  FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for
  more details.

  You should have received a copy of the GNU General Public License along with
  this program; if not, write to the Free Software Foundation, Inc.,
  51 Franklin Street, Fifth Floor, Boston, MA 02110, USA

  The full GNU General Public License is included in this distribution in the
  file called LICENSE.

  Contact Information:
  James P. Ketrenos <ipw2100-admin@linux.intel.com>
  Intel Corporation, 5200 N.E. Elam Young Parkway, Hillsboro, OR 97124-6497

******************************************************************************/

#ifndef __iwl_3945_rs_h__
#define __iwl_3945_rs_h__

#include "iwlwifi_hw.h"

struct ipw_rate_info {
	u8 plcp;
	u8 ieee;
};

#define IPW_FIRST_OFDM_RATE 0
#define IPW_LAST_OFDM_RATE  7
#define IPW_FIRST_CCK_RATE  8
#define IPW_LAST_CCK_RATE   11

#define IPW_OFDM_RATE_6MB_MASK    (1<<0)
#define IPW_OFDM_RATE_9MB_MASK    (1<<1)
#define IPW_OFDM_RATE_12MB_MASK   (1<<2)
#define IPW_OFDM_RATE_18MB_MASK   (1<<3)
#define IPW_OFDM_RATE_24MB_MASK   (1<<4)
#define IPW_OFDM_RATE_36MB_MASK   (1<<5)
#define IPW_OFDM_RATE_48MB_MASK   (1<<6)
#define IPW_OFDM_RATE_54MB_MASK   (1<<7)
#define IPW_CCK_RATE_1MB_MASK     (1<<8)
#define IPW_CCK_RATE_2MB_MASK     (1<<9)
#define IPW_CCK_RATE_5MB_MASK     (1<<10)
#define IPW_CCK_RATE_11MB_MASK    (1<<11)

#define IPW_CCK_BASIC_RATES_MASK    \
       (IPW_CCK_RATE_1MB_MASK     | \
        IPW_CCK_RATE_2MB_MASK)

#define IPW_CCK_RATES_MASK          \
       (IPW_CCK_BASIC_RATES_MASK  | \
        IPW_CCK_RATE_5MB_MASK     | \
        IPW_CCK_RATE_11MB_MASK)

#define IPW_OFDM_BASIC_RATES_MASK   \
        (IPW_OFDM_RATE_6MB_MASK   | \
        IPW_OFDM_RATE_12MB_MASK   | \
        IPW_OFDM_RATE_24MB_MASK)

#define IPW_OFDM_RATES_MASK         \
       (IPW_OFDM_BASIC_RATES_MASK | \
        IPW_OFDM_RATE_9MB_MASK    | \
        IPW_OFDM_RATE_18MB_MASK   | \
        IPW_OFDM_RATE_36MB_MASK   | \
        IPW_OFDM_RATE_48MB_MASK   | \
        IPW_OFDM_RATE_54MB_MASK)

#define IPW_BASIC_RATES_MASK \
        (IPW_OFDM_BASIC_RATES_MASK | \
        IPW_CCK_BASIC_RATES_MASK)

#define IPW_DEFAULT_RATES_MASK ((1<<IPW_MAX_RATES)-1)

#define IPW_INVALID_RATE     0xFF
#define IPW_INVALID_VALUE    -1

#define IPW_MIN_RSSI_VAL                 -100
#define IPW_MAX_RSSI_VAL                    0

extern const struct ipw_rate_info ipw_rates[IPW_MAX_RATES];

static inline int ipw_rate_index_from_plcp(int plcp)
{
	int i = 0;
	for (i = 0; i < ARRAY_SIZE(ipw_rates); i++) 
		if (ipw_rates[i].plcp == plcp)
			return i;
	return -1;
}

struct ipw_priv;
extern int ipw_rate_scale_rxon_handle(struct ipw_priv *priv, s32 sta_id);
extern void ipw_set_rate_scale_handlers(struct rate_control_ops *rs_ops);

#endif
